# nalozimo se lastne funkcije
library(car)
library(blockmodeling)
source("UcbenikR-funkcije.R")
source("LDAandCChelperFunctions.R")
#######################################################################################
#                            PRIPRAVA PODATKOV
#######################################################################################
# uvoz podatkov
# uvoz podatkov
load("../WVSLO_rekodirane", verbose = TRUE)

usmerjenost_vase <- c("V70", "V71", "V73", "V75", "V76")
usmerjenost_ven  <- c("V72", "V74", "V77", "V78", "V79")

Sch <- c(usmerjenost_vase, usmerjenost_ven)

kratka.imena.list <- list("V70" = "ustvarjanost",
                          "V71" = "bogastvo",
                          "V73" = "zabava",
                          "V75" = "uspesnost",
                          "V76" = "pustolovstvo",
                          "V72" = "varnost",
                          "V74" = "dobrodelnost",
                          "V77" = "sprejemljivost",
                          "V78" = "naravovarstvenost",
                          "V79" = "konzervatizem")

# uporabili bomo spremenljivko o zakonskem stanu
table(podatki$status)

# uporabljali bomo samo enote, ki imajo vrednosti pri vseh spremenljivkah
com <- complete.cases(podatki[, c(Sch, "status")])
schData <- podatki[com, c(Sch, "status")]
varLabs <- attributes(podatki)$variable.labels
table(podatki$status[com])
#######################################################################################
#                            PREDPOSTAVKE DISKRIMINANTNE ANALIZE
#######################################################################################
# izracunamo frekvence po kategorijah
# (vsaj dve enoti morata biti v vsaki skupini)
# p < (n-1) (p je stevilo spremenljivk, n je stevilo enot)
(status_t <- table(as.character(schData$status)))
# izracunamo deleze po kategorijah (+ za utemeljitev predpostavljene verjetnosti na populaciji)
prop.table(status_t)

# nobena spremenljivka ne sme biti linearna kombinacija preostalih spremenljivk
round(cor(schData[, Sch]), 2)

# variancno-kovariancna matrika je v vsaki populacijski skupini enaka
# ce je predpostavka krsena, lahko uporabimo kvadratno diskriminantno analizo
# pogledamo tabele
par(mfrow = c(3, 2))
matrike <- by(data = schData[, Sch], INDICES = schData$status, FUN = cor)
for (i in names(matrike)) plot.mat(matrike[[i]], main = i, mar = rep(2, 4), title.line=0, print.x.axis.val = FALSE)
# naredimo test (test ni robusten na odstopanja od normalnosti)
BoxMTest(X = schData[, Sch], cl = as.factor(schData$status), alpha = 0.05, test = FALSE)

# pogledamo povprecja po skupinah glede na odvisno spremenljivko
dev.off()
plotMeans(x=schData[, Sch], by=list(schData[, "status"]), ylim = c(1, 5), ylab="Povprecja", xleg = "bottomright")
# testiramo nicelno domnevo, da so vsa povprecja intervalnh spremenljivk enaka med kategorijami nominalne spremenljivke
summary(manova(as.matrix(schData[, Sch]) ~ schData$status))
# preverimo, med katerimi kategorijami obstajajo razlike v povprecjih
for (i in 1:length(Sch)) {
  a <- summary(aov(schData[, Sch[i]] ~ schData$status))
  cat(Sch[i], "\t F(", a[[1]][1,1], "; ", a[[1]][2,1], ") = ", round(a[[1]][1,4], 3), " \t p = ", round(a[[1]][1,5], 3), "\n", sep = "")
}

#######################################################################################
#                            POMEMBNOST IN STEVILO DISKRIMINANTNIH FUNKCIJ
#######################################################################################
# izvedemo linearno diskriminantno analizo na standardiziranih podatkih
# ker nismo nastavili parametra "prior", funkcija oceni verjetnosti po skupinah iz podatkov
# te vrejtnosti se uporabijo pri klasifikaciji
# CV = cross validation (jackknife)
ldaSch <- ldaPlus(x = schData[, Sch], 
                   grouping = schData$status, 
                   pred = TRUE, 
                   CV = TRUE)
# standardizirane koeficiente dobimo kot
ldaSch$standCoefTotal
# ali
ldaSch$standCoefWithin

# preverimo statisticno znacilnost diskriminatnih funkcij
# Ho: povprecja diskriminantnih funkcij po skupinah so enaka
options(scipen = 999)
round(ldaSch$sigTest, 3)

# pogledamo vrednosti kanonicnih korelacij 
# kako mocno sta povezana sklopa spremenljivk
# (ce gledamo linearni kobinaciji spremenljivk sklopov s takimi 
# vrednostmi regresijskih utezi, da je korelacija maksimalna)
round(ldaSch$eigModel, 3)
#######################################################################################
#                            POMEN DISKRIMINANTNIH FUNKCIJ
#######################################################################################
# najprej nas zanima, med katerimi skupinami locuje posamezna diskriminantna funkcija
round(ldaSch$centroids, 3)
# 1DF:   med porocenimi, vdovelimi ter ostalimi (predvsem med samskimi in vdovelimi)
# 2DF:   med locenimi, porocenimi, izvenzakonskimi ter ostalimi (predvsem med locenimi in vdovelimi)

# sedaj nas zanimajo spremenljivke, ki imajo najvecji vpliv
# na vrednosti diskriminantne funckcije (spremenljivke, ki najbolj locujejo med skupinami)
# ker je bila metoda izvedena na standardzitranih podatkih,
# so to spodaj standaardizirane vrednosti diskriminantne funkcije
(nest.coef <- ldaSch$scaling)
rownames(nest.coef) <-  kratka.imena.list[rownames(nest.coef)]
plot.mat(nest.coef, mar = c(1, 20, 5, 1), main = "")

# za interpretacijo lahko uporabimo tudi korelacije med
# vrednostmi diskriminatne funkcije in pojasnjevalno spremenljivko
korelacije <- cor(schData[, Sch], ldaSch$pred$x)
rownames(korelacije) <- kratka.imena.list[rownames(korelacije)]
plot.mat(korelacije, mar = c(1, 20, 5, 1), main = "")

# pogosto se gledajo "pooled" corelacije zntoraj skupin
korelacije <- ldaSch$corr
rownames(korelacije) <- kratka.imena.list[rownames(korelacije)]
plot.mat(korelacije, mar = c(1, 20, 5, 1), main = "")
#######################################################################################
#                            KVALITETA OCENJENEGA MODELA
#######################################################################################
# pogledamo, kako so organizirani dobljeni rezultati
# enote so v skupine razvrscene na podlagi vrednosti Fisherjeve klasifikacijske funkcije (x.LD1 ... x.LDn)
# izracunane so tudi verjetnosti za pripadnost k posamezni skupini
as.data.frame(ldaSch$pred)[1:10,]
# pogledamo, klasifikacijsko tabelo (frekvence in delezi in delez pravilno razvrscenih) na vse podatkih
ldaSch$class
# pogledamo, klasifikacijsko tabelo (frekvence in delezi in delez pravilno razvrscenih) z uporabo CV
ldaSch$classCV
ldaSch$prior
# za boljso oceno, kako dober je model, je morda dobro pogledati se enkrat prior in dva izracuna na podlagi le-teh:
ldaSch$prior
max(ldaSch$prior)   # delez enot, ki bi jih razvrstili pravilno, ce bi vse uvrstili v najveco skupino
sum(ldaSch$prior^2) # delez enot, ki bi jih razvrstili pravilno, e bi enote razvrscali povsem slucajno

# sedaj predpostavimo, da so na populaciji vse skupine enako velike
# Ali je to v nasem primeru smiselno glede na vedenje o populacijskih vrednostih?
# Kaksne spremembe pricakujemo? Kaksno bo splosno izboljsanje? Kaj pa po skupinah?
ldaSchEqPrior <- update(ldaSch,  prior = rep(1/5, times = 5))
# pogledamo rezultate
ldaSchEqPrior$class
ldaSchEqPrior$classCV

# primerjamo modela
par(mfrow = c(1, 2))
plot.mat(ldaSch$classCV$perTab[,1:length(status_t)], main = "")
plot.mat(ldaSchEqPrior$classCV$perTab[,1:length(status_t)], main = "")

# razlicne velikosti skupin predpostavimo pri izracunu diskriminantne funkcije
# pri klasifikaciji pa prdpostavimo enake velikosti skupin
ldaSchPriorZaIzracun <- ldaPlus(x = schData[, Sch], 
                                 grouping = schData$status, 
                                 # spodnji argument, ce je FALSE, potem se prior verjetnosti
                                 # NE uporabi pri ocenjevanju parametrov
                                 # (namesto tega se uporabi verjetnosti na podlagi vzorca)
                                 usePriorBetweenGroups = FALSE, 
                                 prior = rep(1/5, times = 5),
                                 pred = TRUE, 
                                 CV = TRUE)

# narisemo rezultate in jih primerjamo (koeficiente in klasifikacijske tabele)
par(mfrow = c(2, 3))                               # R = razlicno, E = enako
plot.mat(ldaSch$scaling, main = "")               # R oceno + R za razvrscanje (vsebovano v oceni)
plot.mat(ldaSchEqPrior$scaling, main = "")        # E oceno + E za razvrscanje
plot.mat(ldaSchPriorZaIzracun$scaling, main = "") # R oceno + E za razvrscanje

plot.mat(ldaSch$classCV$perTab[, 1:length(status_t)], main = "")
plot.mat(ldaSchEqPrior$classCV$perTab[, 1:length(status_t)], main = "")
plot.mat(ldaSchPriorZaIzracun$classCV$perTab[, 1:length(status_t)], main = "")

#######################################################################################
#                            ENOTE V PROSTORU PRVIH DVEH DF
#######################################################################################
dev.off()
# narisemo navaden razsevni grfikon - tocke so pobarvane glede na pripadnost k skupini
plot(ldaSch$pred$x[,1:2], 
     col=as.numeric(as.factor(schData$status)), 
     cex=0.5, 
     pch = 16)
# dodamo legendo
legend(x="bottomright", col=1:5, pch=19, legend = levels(as.factor(schData$status)))
# izracunamo vrednosti centroidov
centroidi <- aggregate(ldaSch$pred$x, by = list(schData$status), FUN = mean)
# narisemo centroide
points(centroidi[, 2:3], col = 1:5, cex = 2, pch = 19)
# narisemo osi
abline(h = 0, v = 0)

# sedaj narisemo graf, kjer barve predstavljajo razvrstitev na podlagi modela crke pa dejanske vrednosti
plot(ldaSch$pred$x[,1:2], col=ldaSch$pred$class, 
     pch=c("A", "B", "C", "D")[as.numeric(as.factor(schData$status))], 
     cex=0.5)
# dodamo legendi
legend(x="bottomright", col = 1:5, pch=19, levels(ldaSch$pred$class))
legend(x="bottomleft", pch=c("A", "B", "C", "D"), levels(ldaSch$pred$class))
# izracunamo centroide
# vedno je potrebno pogledati spodnji izpis in 
# prilagoditi col v funkciji points, ker se lahko zgodi,
# da bodo nekatere kategorije prazne in bodo zato barve napacne
centroidi <- aggregate(ldaSch$pred$x, by = list(ldaSch$pred$class), FUN=mean)
points(centroidi[, 2:3], cex=2, pch=19, col = c("red", "green", "blue", 5))
# dodamo osi
abline(h = 0, v = 0)
#######################################################################################
#                            KVADRIDANA DISKRIMINANTNA ANALIZA
#######################################################################################
# izvedemo kvaratno diskriminantno analizo
qdaStatus <- qda(x = scale(schData[, Sch]), grouping = schData$status)

# napovemo pripadnost k skupini (vrednost 'odvisne' spremenljivke)
qdaStatus$pred <- predict(qdaStatus)
# napovemo pripadnost k skupini (vrednost 'odvisne' spremenljivke) na podlagi CV
qdaStatus$predCV <- update(qdaStatus, CV=TRUE)
# naredimo klasifickacijsko tabelo 
tab <- table(pred = qdaStatus$pred$class, obs = schData$status)
prop.table(tab, margin = 2)*100
sum(diag(tab))/sum(tab)*100
# naredimo klasifickacijsko tabelo - CV
tab <- table(pred = qdaStatus$predCV$class, obs = schData$status)
prop.table(tab, margin = 2)*100
sum(diag(tab))/sum(tab)*100

